/// <reference types="socket.io" />
import { Message, MessageWithSeqNumber, ChatClientId, ChatConnectionId, ClientBrowserInfo } from '@stefancfuchs/chat-message-protocol-common';
import ChatServerConnection from './ChatServerConnection';
interface ChatServerOptions {
    clientIdGenerator?: (args?: {
        widgetId: string;
        browserInfo: ClientBrowserInfo;
    }) => Promise<ChatClientId> | ChatClientId;
    clientIdVerifier?: (args?: {
        clientId: ChatClientId;
        browserInfo: ClientBrowserInfo;
    }) => Promise<boolean> | boolean;
    onClientConnected?: (args: {
        clientId: ChatClientId;
        connectionId: ChatConnectionId;
        chatConnection: ChatServerConnection;
        newClient: boolean;
        browserInfo: ClientBrowserInfo;
    }) => void;
    onClientDisconnected?: (args: {
        clientId: ChatClientId;
        connectionId: number;
    }) => void;
    onMessageReceived?: (args: {
        message: MessageWithSeqNumber;
        connectionId: ChatConnectionId;
        clientId: ChatClientId;
    }) => void;
}
export default class ChatServer {
    io: SocketIO.Server;
    protected stopped: boolean;
    protected port: number;
    protected connections: Map<ChatConnectionId, ChatServerConnection>;
    protected nextConnectionId: ChatConnectionId;
    protected nextClientId: number;
    protected clientIdGenerator: ChatServerOptions['clientIdGenerator'];
    protected clientIdVerifier: ChatServerOptions['clientIdVerifier'];
    protected onClientConnected: ChatServerOptions['onClientConnected'];
    protected onClientDisconnected: ChatServerOptions['onClientDisconnected'];
    protected onMessageReceived: ChatServerOptions['onMessageReceived'];
    constructor(port: number, options?: ChatServerOptions);
    getConnection: (index: number) => ChatServerConnection;
    /**
     * Sends a message to client of the connection with the specified ID.
     * @returns The sequence number of the message. Null if the connectionId is invalid
     */
    sendMessage: (message: Message, connectionId: number) => Promise<number>;
    sendBroadcastMessage: (message: Message) => Promise<void>;
    /** Starts the server (server is started by default in constructor) */
    start(): void;
    /** Stops the server */
    stop(): void;
}
export {};
