"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const socketIo = require("socket.io");
// import ReadLinePromised from "./util/ReadLinePromised";
const debug = require('debug')('ChatServer');
// we need to use a lock in order to avoid race conditions with the shared variables.
const AsyncLock = require("async-lock");
var lock = new AsyncLock();
const ChatServerConnection_1 = require("./ChatServerConnection");
class ChatServer {
    // protected onClientRequestNewChat: ChatServerOptions['onClientRequestNewChat']
    constructor(port, options) {
        this.stopped = false;
        this.connections = new Map();
        this.nextConnectionId = 0;
        this.nextClientId = 0;
        // protected handleMessageReceived = (message: MessageWithSeqNumber, connectionId: ChatConnectionId, clientId: ChatClientId) => {
        //   const connection = this.connections.get(connectionId)
        //   if (!connection) return
        //   debug("Message received:")
        //   debug(message)
        //   if (connection.handshakeState === ChatHandshakeState.COMPLETED) {
        //     //process the message
        //     debug("Process message")
        //   }
        // }
        this.getConnection = (index) => {
            return this.connections.get(index);
        };
        /**
         * Sends a message to client of the connection with the specified ID.
         * @returns The sequence number of the message. Null if the connectionId is invalid
         */
        this.sendMessage = (message, connectionId) => __awaiter(this, void 0, void 0, function* () {
            const connection = this.connections.get(connectionId);
            if (!connection) {
                return null;
            }
            return connection.sendMessage(message);
        });
        this.sendBroadcastMessage = (message) => __awaiter(this, void 0, void 0, function* () {
            this.connections.forEach(connection => {
                connection.sendMessage(message);
            });
        });
        const { onClientConnected = undefined, onClientDisconnected = undefined, clientIdGenerator = () => "TEST_ID", clientIdVerifier = () => true, onMessageReceived = undefined, } = options || {};
        this.port = port;
        this.onClientConnected = onClientConnected;
        this.onClientDisconnected = onClientDisconnected;
        this.clientIdGenerator = clientIdGenerator;
        this.clientIdVerifier = clientIdVerifier;
        this.onMessageReceived = onMessageReceived;
        // this.onClientRequestNewChat = onClientRequestNewChat
        this.io = socketIo(port);
        this.io.on('connect', (socket) => __awaiter(this, void 0, void 0, function* () {
            debug('Connected client');
            if (this.stopped) {
                debug("The server is stopped");
                socket.disconnect();
                return;
            }
            let connectionId;
            yield lock.acquire('key', () => {
                //FIXME theoretically an integer overflow can happen here
                connectionId = this.nextConnectionId++;
                const chatConnection = new ChatServerConnection_1.default(socket, {
                    clientIdGenerator,
                    clientIdVerifier,
                    connectionId,
                    onMessageReceived: ({ message, connectionId, clientId }) => {
                        debug(`message received:`);
                        debug(message);
                        this.onMessageReceived && this.onMessageReceived({ message, connectionId, clientId });
                    },
                    onHandshakeCompleted: ({ clientId, connectionId, newClient, browserInfo }) => {
                        this.onClientConnected && this.onClientConnected({ clientId, connectionId, chatConnection, newClient, browserInfo });
                    },
                    /*
                    onClientRequestNewChat: ({ clientId, connectionId }) => {
                      this.onClientRequestNewChat && this.onClientRequestNewChat({ clientId, connectionId, chatConnection })
                    },
                    */
                    // onControlMessageReceived: this.handleControlMessageReceived,
                    onClientDisconnected: ({ clientId, connectionId }) => {
                        this.onClientDisconnected && this.onClientDisconnected({ clientId, connectionId });
                        this.connections.delete(connectionId);
                    }
                });
                this.connections.set(connectionId, chatConnection);
            });
        }));
    }
    /** Starts the server (server is started by default in constructor) */
    start() {
        this.io = socketIo(this.port);
    }
    /** Stops the server */
    stop() {
        this.stopped = true;
        this.io.close();
        console.log(`[ChatServer] Disconnecting ${this.connections.size} clients...`);
        for (const [key, connection] of this.connections) {
            // console.log(`[ChatServer] Disconnecting client ${key}`)
            connection.socket.disconnect();
        }
        // return new Promise<void>(resolve => this.io.close(resolve))
    }
}
exports.default = ChatServer;
//# sourceMappingURL=ChatServer.js.map