import socketIo = require("socket.io");
import { ChatClientId, ChatConnectionId, MessageWithSeqNumber, OutputMessageChannel, ChatHandshakeState, Message, InputMessageChannel, InputControlChannel, OutputControlChannel, ClientBrowserInfo } from "@stefancfuchs/chat-message-protocol-common";
export interface ChatServerConnectionOptions {
    clientIdGenerator: (args?: {
        widgetId: string;
        browserInfo: ClientBrowserInfo;
    }) => Promise<ChatClientId> | ChatClientId;
    clientIdVerifier: (args?: {
        clientId: ChatClientId;
        browserInfo: ClientBrowserInfo;
    }) => Promise<boolean> | boolean;
    connectionId?: ChatConnectionId;
    onMessageReceived?: (args: {
        message: MessageWithSeqNumber;
        connectionId: ChatConnectionId;
        clientId: ChatClientId;
    }) => void;
    onControlMessageReceived?: (args: {
        message: MessageWithSeqNumber;
        connectionId: ChatConnectionId;
        clientId: ChatClientId;
    }) => void;
    maxBufferSize?: number;
    onHandshakeCompleted?: (args: {
        clientId: ChatClientId;
        connectionId: ChatConnectionId;
        newClient: boolean;
        browserInfo: ClientBrowserInfo;
    }) => void;
    onClientDisconnected?: (args: {
        clientId: ChatClientId;
        connectionId: ChatConnectionId;
    }) => void;
}
export default class ChatServerConnection {
    socket: socketIo.Socket;
    protected connectionId?: ChatConnectionId;
    protected chatClientId?: ChatClientId;
    protected inputMessageChannel: InputMessageChannel;
    protected outputMessageChannel: OutputMessageChannel;
    protected inputControlChannel: InputControlChannel;
    protected outputControlChannel: OutputControlChannel;
    protected handshakeState: ChatHandshakeState;
    protected clientIdGenerator: ChatServerConnectionOptions['clientIdGenerator'];
    protected clientIdVerifier: ChatServerConnectionOptions['clientIdVerifier'];
    protected onHandshakeCompleted: ChatServerConnectionOptions['onHandshakeCompleted'];
    protected onClientDisconnected: ChatServerConnectionOptions['onClientDisconnected'];
    constructor(socket: socketIo.Socket, options?: ChatServerConnectionOptions);
    sendMessage: (message: Message) => Promise<number>;
    protected handleControlMessageReceived: (message: MessageWithSeqNumber) => Promise<void>;
}
