"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug = require('debug')('ChatServerConnection');
const AsyncLock = require("async-lock");
const chat_message_protocol_common_1 = require("@stefancfuchs/chat-message-protocol-common");
var lock = new AsyncLock();
class ChatServerConnection {
    constructor(socket, options) {
        this.sendMessage = (message) => __awaiter(this, void 0, void 0, function* () {
            return this.outputMessageChannel.sendMessage(message);
        });
        this.handleControlMessageReceived = (message) => __awaiter(this, void 0, void 0, function* () {
            debug('handleControlMessage start');
            yield lock.acquire('key', () => __awaiter(this, void 0, void 0, function* () {
                debug("Control message received:");
                debug(message);
                switch (message.type) {
                    //The client requested a new ID
                    case chat_message_protocol_common_1.ControlMessageType.CLIENT_REQUEST_NEW_ID:
                        if (this.handshakeState === chat_message_protocol_common_1.ChatHandshakeState.START) {
                            // const widgetId = message.content
                            debug(`1 message.content: ${message.content}`);
                            const contentObj = JSON.parse(message.content);
                            debug(`1 contentObj: ${contentObj}`);
                            const widgetId = contentObj.widgetId;
                            const browserInfo = contentObj.browserInfo;
                            browserInfo.ip = this.socket.handshake.headers['x-forwarded-for'] || this.socket.handshake.address;
                            this.chatClientId = yield this.clientIdGenerator({ widgetId, browserInfo });
                            debug("Generated new client id: " + this.chatClientId);
                            this.outputControlChannel.sendMessage({
                                type: chat_message_protocol_common_1.ControlMessageType.SERVER_ANSWER_NEW_CLIENT_ID,
                                content: this.chatClientId
                            });
                            //TODO wait for ACK from client before completing handshake
                            this.handshakeState = chat_message_protocol_common_1.ChatHandshakeState.COMPLETED;
                            this.outputMessageChannel.startSendingMessages();
                            this.onHandshakeCompleted && this.onHandshakeCompleted({ clientId: this.chatClientId, connectionId: this.connectionId, newClient: true, browserInfo });
                        }
                        break;
                    //The client has an existing ID
                    case chat_message_protocol_common_1.ControlMessageType.CLIENT_HAS_EXISTING_ID:
                        if (this.handshakeState === chat_message_protocol_common_1.ChatHandshakeState.START) {
                            //Validate the client ID
                            // const clientId = message.content
                            debug(`1 message.content: ${message.content}`);
                            const contentObj = JSON.parse(message.content);
                            debug(`1 contentObj: ${contentObj}`);
                            const clientId = contentObj.clientId;
                            const browserInfo = contentObj.browserInfo;
                            browserInfo.ip = this.socket.handshake.headers['x-forwarded-for'] || this.socket.handshake.address;
                            if (yield this.clientIdVerifier({ clientId, browserInfo })) {
                                debug("client ID is valid");
                                this.outputControlChannel.sendMessage({
                                    type: chat_message_protocol_common_1.ControlMessageType.SERVER_ANSWER_CLIENT_ID_VALID,
                                    content: String(true) // tell the client the id is valid.
                                });
                                this.chatClientId = clientId;
                                this.handshakeState = chat_message_protocol_common_1.ChatHandshakeState.COMPLETED;
                                this.outputMessageChannel.startSendingMessages();
                                this.onHandshakeCompleted && this.onHandshakeCompleted({ clientId: this.chatClientId, connectionId: this.connectionId, newClient: false, browserInfo });
                            }
                            else {
                                debug("client ID is invalid");
                                this.outputControlChannel.sendMessage({
                                    type: chat_message_protocol_common_1.ControlMessageType.SERVER_ANSWER_CLIENT_ID_VALID,
                                    content: String(false) // tell the client the id is invalid. It will then ask for a new one.
                                });
                            }
                        }
                        break;
                    // client is requesting a new chat
                    /*
                    case ControlMessageType.CLIENT_REQUEST_NEW_CHAT:
                      const contentObj: ClientControlMessageContent = JSON.parse(message.content)
                      const widgetId = contentObj.widgetId
                      const browserInfo = contentObj.browserInfo
                      this.onClientRequestNewChat && this.onClientRequestNewChat({ clientId: this.chatClientId, connectionId: this.connectionId, widgetId, browserInfo })
                      break
                    */
                }
            }));
        });
        const { clientIdGenerator = undefined, clientIdVerifier = undefined, connectionId = undefined, onMessageReceived = undefined, onControlMessageReceived = undefined, maxBufferSize = undefined, 
        // onMessageAck = undefined,
        onHandshakeCompleted = undefined, 
        // onClientRequestNewChat = undefined,
        onClientDisconnected = undefined, } = options || {};
        this.socket = socket;
        this.handshakeState = chat_message_protocol_common_1.ChatHandshakeState.START;
        this.clientIdGenerator = clientIdGenerator;
        this.clientIdVerifier = clientIdVerifier;
        this.onHandshakeCompleted = onHandshakeCompleted;
        this.connectionId = connectionId;
        // this.onClientRequestNewChat = onClientRequestNewChat
        this.onClientDisconnected = onClientDisconnected;
        //creates the message channels
        this.inputMessageChannel = new chat_message_protocol_common_1.InputMessageChannel(socket, {
            connectionId,
            onMessageReceived: (message, connectionId) => {
                if (this.handshakeState === chat_message_protocol_common_1.ChatHandshakeState.COMPLETED) {
                    onMessageReceived({ message, connectionId, clientId: this.chatClientId });
                }
            }
        });
        this.outputMessageChannel = new chat_message_protocol_common_1.OutputMessageChannel(socket, { connectionId, maxBufferSize });
        this.outputMessageChannel.stopSendingMessages();
        //creates the control channels
        this.inputControlChannel = new chat_message_protocol_common_1.InputControlChannel(socket, {
            connectionId,
            onMessageReceived: (message, connectionId) => {
                this.handleControlMessageReceived(message);
                onControlMessageReceived && onControlMessageReceived({ message, connectionId, clientId: this.chatClientId });
            }
        });
        this.outputControlChannel = new chat_message_protocol_common_1.OutputControlChannel(socket, { connectionId, maxBufferSize });
        socket.on('disconnect', () => {
            lock.acquire('key', () => {
                debug(`Client ${this.chatClientId} disconnected`);
                onClientDisconnected && onClientDisconnected({ clientId: this.chatClientId, connectionId: this.connectionId });
            });
        });
    }
}
exports.default = ChatServerConnection;
//# sourceMappingURL=ChatServerConnection.js.map